@@ -1,17 +1,49 @@
 """
-OAuth 2.0 Grant types
+Grants are the heart of OAuth 2.0. Each Grant defines one way for a client to
+retrieve an authorization. They are defined in
+`Section 4 <http://tools.ietf.org/html/rfc6749#section-4>`_ of the OAuth 2.0
+spec.
+
+OAuth 2.0 comes in two flavours of how an access token is issued:
+two-legged and three-legged auth. To avoid confusion they are explained in
+short here.
+
+Three-legged OAuth
+------------------
+The "three" symbolizes the parties that are involved:
+
+* The client that wants to access a resource on behalf of the user.
+* The user who grants access to her resources.
+* The server that issues the access token if the user allows it.
+
+Two-legged OAuth
+----------------
+The two-legged OAuth process differs from the three-legged process by one
+missing paricipant. The user cannot allow or deny access.
+
+So there are two remaining parties:
+
+* The client that wants to access a resource.
+* The server that issues the access.
+
 """
-import datetime
-from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError
+from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError,\
+    ClientNotFoundError, UserNotAuthenticated
 from oauth2.compatibility import urlencode, quote
 import json
+import time
+from oauth2 import AuthorizationCode, AccessToken
 
 class Scope(object):
     """
-    Handler of scopes in the oauth2 flow.
+    Handling of the "scope" parameter in a request.
     
-    :param available: A list of strings each defining one supported scope
-    :param default: Fallback value in case no scope is present in request
+    If ``available`` and ``default`` are both ``None``, the "scope" parameter
+    is ignored (the default).
+    
+    :param available: A list of strings each defining one supported scope.
+    :param default: Value to fall back to in case no scope is present in a
+                    request.
     """
     def __init__(self, available=None, default=None):
         self.scopes     = []
@@ -26,9 +58,18 @@ class Scope(object):
     
     def parse(self, request):
         """
-        Parses scope values from given request.
+        Parses scope value in given request.
         
-        :param request: ``oauth2.web.Request``
+        Expects the value of the "scope" parameter in request to be a string
+        where each requested scope is separated by a white space::
+        
+            # One scope requested
+            "profile_read"
+            
+            # Multiple scopes
+            "profile_read profile_write"
+        
+        :param request: An instance of :class:`oauth2.web.Request`.
         """
         req_scope = request.get_param("scope")
         
@@ -53,8 +94,16 @@ class Scope(object):
 
 class ScopeGrant(object):
     """
-    Helper class to be extended by Grant classes that support the "scope"
-    parameter in request.
+    Handling of scopes in the OAuth 2.0 flow.
+    
+    Inherited by all grants that need to support scopes.
+    
+    :param default_scope: The scope identifier that is returned by default.
+                          (optional)
+    :param scopes:        A list of strings identifying the scopes that the
+                          grant supports.
+    :param scope_class: The class that does the actual handling in a request.
+                        Default: :class:`oauth2.grant.Scope`.
     """
     def __init__(self, default_scope=None, scopes=None, scope_class=Scope):
         self.default_scope = default_scope
@@ -96,6 +145,8 @@ class GrantHandler(object):
 class GrantHandlerFactory(object):
     """
     Base class every handler factory can extend.
+    
+    This class defines the basic interface of each Grant.
     """
     def __call__(self, request, server):
         raise NotImplementedError
@@ -129,20 +180,20 @@ class AuthRequestMixin(object):
                                   explanation="Missing client_id parameter")
         self.client_id = client_id
         
-        client_data = self.client_store.fetch_by_client_id(self.client_id)
-        
-        if client_data is None:
+        try:
+            client_data = self.client_store.fetch_by_client_id(self.client_id)
+        except ClientNotFoundError:
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="No client registered")
         
         redirect_uri = request.get_param("redirect_uri")
         
         if (redirect_uri is not None
-            and redirect_uri not in client_data["redirect_uris"]):
+            and client_data.has_redirect_uri(redirect_uri) == False):
             raise OAuthInvalidError(error="invalid_request",
                                     explanation="redirect_uri is not registered for this client")
         else:
-            self.redirect_uri = client_data["redirect_uris"][0]
+            self.redirect_uri = client_data.redirect_uris[0]
         
         self.state = request.get_param("state")
         
@@ -160,7 +211,7 @@ class AuthorizationCodeAuthHandler(AuthRequestMixin, GrantHandler):
     
     def __init__(self, auth_token_store, client_store, scope_handler,
                  site_adapter, token_generator):
-        self.auth_token_store = auth_token_store
+        self.auth_code_store = auth_token_store
         
         AuthRequestMixin.__init__(self, client_store, scope_handler,
                                   site_adapter, token_generator)
@@ -172,24 +223,28 @@ class AuthorizationCodeAuthHandler(AuthRequestMixin, GrantHandler):
         A form to authorize the access of the application can be displayed with
         the help of `oauth2.web.SiteAdapter`.
         """
-        user_data = self.site_adapter.authenticate(request, environ)
-        
-        if user_data is None:
+        try:
+            user_data = self.site_adapter.authenticate(request, environ,
+                                                       self.scope_handler.scopes)
+        except UserNotAuthenticated:
             return self.site_adapter.render_auth_page(request, response,
                                                       environ,
                                                       self.scope_handler.scopes)
         
         code = self.token_generator.generate()
-        current_time = datetime.datetime.now()
-        expiration_delta = datetime.timedelta(seconds=self.token_expiration)
-        expires = current_time + expiration_delta
-        self.auth_token_store.save_code(self.client_id, code, expires,
-                                        self.redirect_uri,
-                                        self.scope_handler.scopes, user_data)
+        expires = int(time.time()) + self.token_expiration
+        
+        auth_code = AuthorizationCode(client_id=self.client_id, code=code,
+                                      expires_at=expires,
+                                      redirect_uri=self.redirect_uri,
+                                      scopes=self.scope_handler.scopes,
+                                      data=user_data)
+        
+        self.auth_code_store.save_code(auth_code)
         
         response.add_header("Location", self._generate_location(code))
         response.body = ""
-        response.status_code = "302 Found"
+        response.status_code = 302
         
         return response
     
@@ -203,7 +258,7 @@ class AuthorizationCodeAuthHandler(AuthRequestMixin, GrantHandler):
         
         location = "%s?%s" % (self.redirect_uri, query)
         
-        response.status_code = "302 Found"
+        response.status_code = 302
         response.body = ""
         response.add_header("Location", location)
         
@@ -234,7 +289,7 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         self.scopes        = []
         
         self.access_token_store = access_token_store
-        self.auth_token_store   = auth_token_store
+        self.auth_code_store   = auth_token_store
         self.client_store       = client_store
         self.token_generator    = token_generator
     
@@ -265,16 +320,17 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         
         Calls `oauth2.store.AccessTokenStore` to persist the token.
         """
-        access_token = self.token_generator.generate()
+        token = self.token_generator.generate()
         
-        result = {"access_token": access_token, "token_type": "Bearer"}
+        result = {"access_token": token, "token_type": "Bearer"}
         
-        self.access_token_store.save_token(client_id=self.client_id,
-                                           scopes=self.scopes,
-                                           token=access_token, user_data={})
+        access_token = AccessToken(client_id=self.client_id, token=token,
+                                   scopes=self.scopes)
+        
+        self.access_token_store.save_token(access_token)
         
         response.body = json.dumps(result)
-        response.status_code = "200 OK"
+        response.status_code = 200
         
         response.add_header("Content-type", "application/json")
         
@@ -287,7 +343,7 @@ class AuthorizationCodeTokenHandler(GrantHandler):
         """
         msg = {"error": error.error}
         
-        response.status_code = "400 Bad Request"
+        response.status_code = 400
         response.add_header("Content-type", "application/json")
         response.body = json.dumps(msg)
         
@@ -308,61 +364,61 @@ class AuthorizationCodeTokenHandler(GrantHandler):
                                               "in request")
     
     def _validate_client(self):
-        client = self.client_store.fetch_by_client_id(self.client_id)
-        
-        if client is None:
+        try:
+            client = self.client_store.fetch_by_client_id(self.client_id)
+        except ClientNotFoundError:
             raise OAuthClientError(error="invalid_client",
                                    explanation="Unknown client")
         
-        if client["secret"] != self.client_secret:
+        if client.secret != self.client_secret:
             raise OAuthClientError(error="invalid_client",
                                    explanation="Invalid client_secret")
         
-        if self.redirect_uri not in client["redirect_uris"]:
+        if client.has_redirect_uri(self.redirect_uri) == False:
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Invalid redirect_uri parameter")
     
     def _validate_code(self):
-        stored_code = self.auth_token_store.fetch_by_code(self.code)
+        stored_code = self.auth_code_store.fetch_by_code(self.code)
         
         if stored_code is None:
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Invalid authorization code " \
                                               "parameter")
         
-        if stored_code["code"] != self.code:
+        if stored_code.code != self.code:
             raise OAuthInvalidError(error="invalid_grant",
                                   explanation="Invalid code parameter in " \
                                               "request")
         
-        if stored_code["redirect_uri"] != self.redirect_uri:
+        if stored_code.redirect_uri != self.redirect_uri:
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Invalid redirect_uri parameter")
         
-        current_time = datetime.datetime.now()
-        if current_time > stored_code["expired_at"]:
+        if stored_code.is_expired():
             raise OAuthInvalidError(error="invalid_grant",
                                   explanation="Authorization code has expired")
         
-        self.scopes = stored_code["scopes"]
+        self.scopes = stored_code.scopes
 
 class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
     """
-    Implementation of the Authorization Code Grant auth flow also known as
-    "three-legged auth".
+    Implementation of the Authorization Code Grant auth flow.
     
-    Register an instance of this class with `oauth2.AuthorizationServer`
-    like this::
+    This is a three-legged OAuth process.
     
-        auth_server = AuthorizationServer()
+    Register an instance of this class with
+    :class:`oauth2.AuthorizationController` like this::
+    
+        auth_controller = AuthorizationController()
         
-        auth_server.add_grant_type(AuthorizationCodeGrant())
+        auth_controller.add_grant_type(AuthorizationCodeGrant())
     """
     def __call__(self, request, server):
         if (request.post_param("grant_type") == "authorization_code"
             and request.path == server.token_path):
             return AuthorizationCodeTokenHandler(server.access_token_store,
-                                                 server.auth_token_store,
+                                                 server.auth_code_store,
                                                  server.client_store,
                                                  server.token_generator)
         
@@ -370,7 +426,7 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
             and request.path == server.authorize_path):
             scope_handler = self._create_scope_handler()
             
-            return AuthorizationCodeAuthHandler(server.auth_token_store,
+            return AuthorizationCodeAuthHandler(server.auth_code_store,
                                                 server.client_store,
                                                 scope_handler,
                                                 server.site_adapter,
@@ -380,15 +436,16 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
 
 class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
     """
-    Implementation of the Implicit Grant auth flow also known as
-    "two-legged auth".
+    Implementation of the Implicit Grant auth flow.
     
-    Register an instance of this class with `oauth2.AuthorizationServer`
-    like this::
+    This is a three-legged OAuth process.
+
+    Register an instance of this class with
+    :class:`oauth2.AuthorizationController` like this::
     
-        auth_server = AuthorizationController()
+        auth_controller = AuthorizationController()
         
-        auth_server.add_grant_type(ImplicitGrant())
+        auth_controller.add_grant_type(ImplicitGrant())
     """
     def __call__(self, request, server):
         response_type = request.get_param("response_type")
@@ -416,18 +473,21 @@ class ImplicitGrantHandler(AuthRequestMixin, GrantHandler):
             raise OAuthUserError(error="access_denied",
                                  explanation="Authorization denied by user")
         
-        user_data = self.site_adapter.authenticate(request, environ)
-        
-        if user_data is None:
+        try:
+            user_data = self.site_adapter.authenticate(request, environ,
+                                                       self.scope_handler.scopes)
+        except UserNotAuthenticated:
             return self.site_adapter.render_auth_page(request, response,
                                                       environ,
                                                       self.scope_handler.scopes)
         
         token = self.token_generator.generate()
         
-        self.access_token_store.save_token(client_id=self.client_id,
-                                           scopes=self.scope_handler.scopes,
-                                           token=token, user_data=user_data)
+        access_token = AccessToken(client_id=self.client_id, token=token,
+                                   data=user_data,
+                                   scopes=self.scope_handler.scopes)
+        
+        self.access_token_store.save_token(access_token)
         
         return self._redirect_access_token(response, token)
     
@@ -436,7 +496,7 @@ class ImplicitGrantHandler(AuthRequestMixin, GrantHandler):
         
         response.add_header("Location", redirect_location)
         response.body = ""
-        response.status_code = "302 Moved Temporarily"
+        response.status_code = 302
         
         return response
     
@@ -449,7 +509,7 @@ class ImplicitGrantHandler(AuthRequestMixin, GrantHandler):
         if self.scope_handler.send_back is True:
             uri_with_fragment += "&scope=" + "%20".join(self.scope_handler.scopes)
         
-        response.status_code = "302 Moved Temporarily"
+        response.status_code = 302
         response.add_header("Location", uri_with_fragment)
         response.content = ""
         
@@ -457,8 +517,18 @@ class ImplicitGrantHandler(AuthRequestMixin, GrantHandler):
 
 class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant):
     """
-    Factory class to return a ResourceOwnerGrantHandler if the incoming
-    request matches the conditions for this type of request.
+    Implementation of the Resource Owner Password Credentials Grant auth flow.
+    
+    In this Grant a user provides a user name and a password.
+    An access token is issued if the auth server was able to verify the user
+    by her credentials. 
+    
+    Register an instance of this class with
+    :class:`oauth2.AuthorizationController` like this::
+    
+        auth_controller = AuthorizationController()
+        
+        auth_controller.add_grant_type(ResourceOwnerGrant())
     """
     def __call__(self, request, server):
         """
@@ -499,21 +569,24 @@ class ResourceOwnerGrantHandler(GrantHandler):
         it and issues a new access token that is returned to the client on
         successful validation.
         """
-        user_data = self.site_adapter.authenticate(request, environ)
+        user_data = self.site_adapter.authenticate(request, environ,
+                                                   self.scope_handler.scopes)
         
-        access_token = self.token_generator.generate()
+        token = self.token_generator.generate()
         
-        self.access_token_store.save_token(self.client_id, access_token,
-                                           self.scope_handler.scopes,
-                                           user_data)
+        access_token = AccessToken(client_id=self.client_id, token=token,
+                                   data=user_data,
+                                   scopes=self.scope_handler.scopes)
         
-        response_body = {"access_token": access_token, "token_type": "bearer"}
+        self.access_token_store.save_token(access_token)
+        
+        response_body = {"access_token": token, "token_type": "bearer"}
         
         if self.scope_handler.send_back is True:
             response_body["scope"] = " ".join(self.scope_handler.scopes)
         
         response.add_header("Content-Type", "application/json")
-        response.status_code = "200 OK"
+        response.status_code = 200
         response.body = json.dumps(response_body)
         
         return response
@@ -528,13 +601,13 @@ class ResourceOwnerGrantHandler(GrantHandler):
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Missing client_id parameter")
         
-        client = self.client_store.fetch_by_client_id(self.client_id)
-        
-        if client is None:
+        try:
+            client = self.client_store.fetch_by_client_id(self.client_id)
+        except ClientNotFoundError:
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Unknown client")
         
-        if client["secret"] != request.post_param("client_secret"):
+        if client.secret != request.post_param("client_secret"):
             raise OAuthInvalidError(error="invalid_request",
                                   explanation="Could not authenticate client")
         
@@ -551,7 +624,7 @@ class ResourceOwnerGrantHandler(GrantHandler):
         """
         msg = {"error": error.error, "description": error.explanation}
         
-        response.status_code = "400 Bad Request"
+        response.status_code = 400
         response.add_header("Content-Type", "application/json")
         response.body = json.dumps(msg)
         
