@@ -6,18 +6,18 @@ from oauth2.grant import ImplicitGrantHandler, AuthorizationCodeAuthHandler,\
     AuthRequestMixin, AuthorizationCodeTokenHandler, ImplicitGrant,\
     AuthorizationCodeGrant, ResourceOwnerGrantHandler, ResourceOwnerGrant,\
     Scope
-from oauth2.store import ClientStore, AuthTokenStore, AccessTokenStore
-from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError
+from oauth2.store import ClientStore, AuthCodeStore, AccessTokenStore
+from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError,\
+    ClientNotFoundError, UserNotAuthenticated
+from oauth2 import AuthorizationController, Client, AuthorizationCode,\
+    AccessToken
 
 # Mock datetime.now()
 # see http://stackoverflow.com/questions/4481954/python-trying-to-mock-datetime-date-today-but-not-working
-import datetime
-from oauth2 import AuthorizationController
-class DatetimeMock(datetime.datetime):
-    @classmethod
-    def now(cls):
-        return cls(1990, 1, 1, 0, 0, 0)
-datetime.datetime = DatetimeMock
+import time
+def time():
+    return 1000
+time.time = time
 
 class AuthorizationCodeGrantTestCase(unittest.TestCase):
     def test_create_auth_handler(self):
@@ -36,7 +36,7 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         
         server_mock = Mock()
         server_mock.authorize_path = path
-        server_mock.auth_token_store = Mock()
+        server_mock.auth_code_store = Mock()
         server_mock.client_store = Mock()
         server_mock.site_adapter = Mock()
         server_mock.token_generator = Mock()
@@ -61,7 +61,7 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         server_mock.authorize_path = "/auth"
         server_mock.token_path = path
         server_mock.access_token_store = Mock(spec=AccessTokenStore)
-        server_mock.auth_token_store = Mock()
+        server_mock.auth_code_store = Mock()
         server_mock.client_store = Mock()
         server_mock.token_generator = Mock()
         
@@ -93,13 +93,16 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         redirect_uri = "http://somewhere"
         state        = "state"
         
+        client_mock = Mock(Client)
+        client_mock.redirect_uris = [redirect_uri]
+        
         request_mock = Mock(spec=Request)
         request_mock.get_param.side_effect = [client_id, redirect_uri, state]
         
         scope_handler_mock = Mock(Scope)
         
         clientStoreMock = Mock(spec=ClientStore)
-        clientStoreMock.fetch_by_client_id.return_value = {"redirect_uris": [redirect_uri]}
+        clientStoreMock.fetch_by_client_id.return_value = client_mock
         
         handler = AuthRequestMixin(client_store=clientStoreMock,
                                    site_adapter=Mock(),
@@ -138,7 +141,7 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         self.assertEqual(e.error, "invalid_request")
         self.assertEqual(e.explanation, "Missing client_id parameter")
     
-    def test_read_validate_params_invalid_client_id(self):
+    def test_read_validate_params_unknown_client_id(self):
         """
         AuthRequestMixin.read_validate_params should raise an OAuthInvalidError if no client with given client_id exists
         """
@@ -148,7 +151,7 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         request_mock.get_param.return_value = client_id
         
         clientStoreMock = Mock(spec=ClientStore)
-        clientStoreMock.fetch_by_client_id.return_value = None
+        clientStoreMock.fetch_by_client_id.side_effect = ClientNotFoundError
         
         handler = AuthRequestMixin(client_store=clientStoreMock,
                                    scope_handler=Mock(), site_adapter=Mock(),
@@ -171,11 +174,14 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         client_id = "abc"
         redirect_uri = "http://endpoint-one"
         
+        client_mock = Mock(Client)
+        client_mock.has_redirect_uri.return_value = False
+        
         request_mock = Mock(spec=Request)
         request_mock.get_param.side_effect = [client_id, redirect_uri]
         
         clientStoreMock = Mock(spec=ClientStore)
-        clientStoreMock.fetch_by_client_id.return_value = {"redirect_uris": ["http://endpoint-two"]}
+        clientStoreMock.fetch_by_client_id.return_value = client_mock
         
         handler = AuthRequestMixin(client_store=clientStoreMock,
                                    scope_handler=Mock(), site_adapter=Mock(),
@@ -204,7 +210,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         
         location_uri = "%s?code=%s&state=%s" % (redirect_uri, code, state)
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
         
         response_mock = Mock(spec=Response)
         
@@ -221,7 +227,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         token_generator_mock.generate.return_value = code
         
         handler = AuthorizationCodeAuthHandler(
-            auth_token_store=auth_token_store_mock,
+            auth_token_store=auth_code_store_mock,
             client_store=Mock(), scope_handler=scope_handler_mock,
             site_adapter=site_adapter_mock,
             token_generator=token_generator_mock
@@ -234,12 +240,9 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         
         token_generator_mock.generate.assert_called_with()
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ)
-        auth_token_store_mock.save_code.assert_called_with(
-            client_id, code, DatetimeMock(1990, 1, 1, 0, 10, 0), redirect_uri,
-            scopes, user_data
-        )
-        self.assertEqual(response.status_code, "302 Found")
+                                                          environ, scopes)
+        self.assertTrue(auth_code_store_mock.save_code.called)
+        self.assertEqual(response.status_code, 302)
         self.assertEqual(response.body, "")
         response_mock.add_header.assert_called_with("Location", location_uri)
     
@@ -268,7 +271,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         token_generator_mock.generate.return_value = code
         
         handler = AuthorizationCodeAuthHandler(
-            auth_token_store=Mock(spec=AuthTokenStore),
+            auth_token_store=Mock(spec=AuthCodeStore),
             client_store=Mock(), scope_handler=scope_handler_mock,
             site_adapter=site_adapter_mock,
             token_generator=token_generator_mock
@@ -280,7 +283,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         response = handler.process(Mock(spec=Request), response_mock, environ)
         
         token_generator_mock.generate.assert_called_with()
-        self.assertEqual(response.status_code, "302 Found")
+        self.assertEqual(response.status_code, 302)
         self.assertEqual(response.body, "")
         response_mock.add_header.assert_called_with("Location", location_uri)
     
@@ -298,7 +301,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         
         site_adapter_mock = Mock(spec=SiteAdapter)
-        site_adapter_mock.authenticate.return_value = None
+        site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
         site_adapter_mock.render_auth_page.return_value = response_mock
         
         handler = AuthorizationCodeAuthHandler(
@@ -334,7 +337,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Location",
                                                     expected_redirect)
-        response_mock.status_code = "302 Found"
+        response_mock.status_code = 302
         response_mock.body = ""
         self.assertEqual(result, response_mock)
     
@@ -347,26 +350,28 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         redirect_uri = "http://callback"
         scopes = ["scope"]
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
-        auth_token_store_mock.fetch_by_code.return_value = {
-            "code": code,
-            "expired_at": DatetimeMock(1990, 1, 1, 0, 0, 0),
-            "redirect_uri": redirect_uri,
-            "scopes": scopes
-        }
+        auth_code = Mock(AuthorizationCode)
+        auth_code.code = code
+        auth_code.is_expired.return_value = False
+        auth_code.redirect_uri = redirect_uri
+        auth_code.scopes = scopes
+        
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
+        auth_code_store_mock.fetch_by_code.return_value = auth_code
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.redirect_uris = [redirect_uri]
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret, code,
                                                redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                auth_token_store_mock,
+                                                auth_code_store_mock,
                                                 client_store_mock, Mock())
         result = handler.read_validate_params(request_mock)
         
@@ -374,7 +379,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
                                                   call("client_secret"),
                                                   call("code"),
                                                   call("redirect_uri")])
-        auth_token_store_mock.fetch_by_code.assert_called_with(code)
+        auth_code_store_mock.fetch_by_code.assert_called_with(code)
         self.assertEqual(handler.client_id, client_id)
         self.assertEqual(handler.client_secret, client_secret)
         self.assertEqual(handler.code, code)
@@ -388,17 +393,18 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         code = None
         redirect_uri = "http://callback"
         
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret, code,
                                                redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                Mock(spec=AuthTokenStore),
+                                                Mock(spec=AuthCodeStore),
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -417,23 +423,25 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         code_actual   = "xyz"
         redirect_uri  = "http://callback"
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
-        auth_token_store_mock.fetch_by_code.return_value = {
-            "code": code_expected
-        }
+        auth_code_mock = Mock(AuthorizationCode)
+        auth_code_mock.code = code_expected
+        
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
+        auth_code_store_mock.fetch_by_code.return_value = auth_code_mock
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.redirect_uris = [redirect_uri]
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
                                                code_actual, redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                auth_token_store_mock,
+                                                auth_code_store_mock,
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -451,14 +459,14 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         redirect_uri  = "http://callback"
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = None
+        client_store_mock.fetch_by_client_id.side_effect = ClientNotFoundError
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret, code,
                                                redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                Mock(spec=AuthTokenStore),
+                                                Mock(spec=AuthCodeStore),
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthClientError) as expected:
@@ -476,17 +484,19 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         code   = "xyz"
         redirect_uri  = "http://callback"
         
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret_expected
+        client_mock.redirect_uris = [redirect_uri]
+        
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "secret": client_secret_expected
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret_actual,
                                                code, redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                Mock(spec=AuthTokenStore),
+                                                Mock(spec=AuthCodeStore),
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthClientError) as expected:
@@ -501,21 +511,21 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         client_id     = "abc"
         client_secret = "t%gH"
         code   = "xyz"
-        redirect_uri_actual = "http://invalid-callback"
-        redirect_uri_expected = "http://callback"
+        redirect_uri = "http://invalid-callback"
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.has_redirect_uri.return_value = False
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri_expected],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
-                                               code, redirect_uri_actual]
+                                               code, redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                Mock(spec=AuthTokenStore),
+                                                Mock(spec=AuthCodeStore),
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -532,21 +542,22 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         code   = "xyz"
         redirect_uri = "http://callback"
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
-        auth_token_store_mock.fetch_by_code.return_value = None
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
+        auth_code_store_mock.fetch_by_code.return_value = None
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.redirect_uris = [redirect_uri]
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
                                                code, redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                auth_token_store_mock,
+                                                auth_code_store_mock,
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -565,24 +576,26 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         redirect_uri_actual = "http://invalid-callback"
         redirect_uri_expected = "http://callback"
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
-        auth_token_store_mock.fetch_by_code.return_value = {
-            "code": code,
-            "redirect_uri": redirect_uri_actual
-        }
+        auth_code_mock = Mock(AuthorizationCode)
+        auth_code_mock.code = code
+        auth_code_mock.redirect_uri = redirect_uri_actual
+        
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
+        auth_code_store_mock.fetch_by_code.return_value = auth_code_mock
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.redirect_uris = [redirect_uri_expected]
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri_expected],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
                                                code, redirect_uri_expected]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                auth_token_store_mock,
+                                                auth_code_store_mock,
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -598,27 +611,28 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         client_secret = "t%gH"
         code   = "xyz"
         redirect_uri = "http://callback"
-        token_expired_at = DatetimeMock(1989, 12, 31, 23, 59, 59)
         
-        auth_token_store_mock = Mock(spec=AuthTokenStore)
-        auth_token_store_mock.fetch_by_code.return_value = {
-            "code": code,
-            "expired_at": token_expired_at,
-            "redirect_uri": redirect_uri
-        }
+        auth_code_mock = Mock(AuthorizationCode)
+        auth_code_mock.code = code
+        auth_code_mock.redirect_uri = redirect_uri
+        auth_code_mock.is_expired.return_value = True
+        
+        auth_code_store_mock = Mock(spec=AuthCodeStore)
+        auth_code_store_mock.fetch_by_code.return_value = auth_code_mock
+        
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        client_mock.redirect_uris = [redirect_uri]
         
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {
-            "redirect_uris": [redirect_uri],
-            "secret": client_secret
-        }
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(spec=Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
                                                code, redirect_uri]
         
         handler = AuthorizationCodeTokenHandler(Mock(spec=AccessTokenStore),
-                                                auth_token_store_mock,
+                                                auth_code_store_mock,
                                                 client_store_mock, Mock())
         
         with self.assertRaises(OAuthInvalidError) as expected:
@@ -646,18 +660,16 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         response_mock.status_code = None
         
         handler = AuthorizationCodeTokenHandler(access_token_store_mock,
-                                                Mock(spec=AuthTokenStore),
+                                                Mock(spec=AuthCodeStore),
                                                 Mock(spec=ClientStore),
                                                 token_generator_mock)
         handler.client_id = client_id
         handler.scopes    = scopes
         response = handler.process(Mock(spec=Request), response_mock, {})
         
-        access_token_store_mock.save_token.assert_called_with(client_id=client_id,
-                                                              scopes=scopes,
-                                                              token=access_token,
-                                                              user_data={})
-        self.assertEqual(response.status_code, "200 OK")
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(response.status_code, 200)
         self.assertEqual(response.body, json.dumps(expected_body))
         response_mock.add_header.assert_called_with("Content-type",
                                                     "application/json")
@@ -676,7 +688,7 @@ class AuthorizationCodeTokenHandlerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Content-type",
                                                     "application/json")
-        self.assertEqual(response_mock.status_code, "400 Bad Request")
+        self.assertEqual(response_mock.status_code, 400)
         self.assertEqual(response_mock.body, json.dumps(expected_content))
         self.assertEqual(result, response_mock)
 
@@ -753,14 +765,14 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         result_response = handler.process(request_mock, responseMock, environ)
         
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ)
-        access_token_store_mock.save_token.assert_called_with(client_id=client_id,
-                                                              scopes=scopes,
-                                                              token=token,
-                                                              user_data=user_id)
+                                                          environ, scopes)
+        
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
+        
         responseMock.add_header.assert_called_with("Location",
                                                    redirect_uri_with_token)
-        self.assertEqual(responseMock.status_code, "302 Moved Temporarily")
+        self.assertEqual(responseMock.status_code, 302)
         self.assertEqual(responseMock.content, "")
         self.assertEqual(result_response, responseMock)
     
@@ -803,7 +815,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Location",
                                                    expected_redirect_uri)
-        self.assertEqual(response_mock.status_code, "302 Moved Temporarily")
+        self.assertEqual(response_mock.status_code, 302)
         self.assertEqual(response_mock.content, "")
         self.assertEqual(result_response, response_mock)
     
@@ -840,7 +852,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Location",
                                                    expected_redirect_uri)
-        self.assertEqual(response_mock.status_code, "302 Moved Temporarily")
+        self.assertEqual(response_mock.status_code, 302)
         self.assertEqual(response_mock.content, "")
         self.assertEqual(result_response, response_mock)
     
@@ -857,7 +869,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         
         site_adapter_mock = Mock(spec=SiteAdapter)
-        site_adapter_mock.authenticate.return_value = None
+        site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
         site_adapter_mock.render_auth_page.return_value = confirmation_page_result
         
         handler = ImplicitGrantHandler(
@@ -868,7 +880,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         result_response = handler.process(request_mock, responseMock, environ)
         
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ)
+                                                          environ, scopes)
         site_adapter_mock.render_auth_page.assert_called_with(request_mock,
                                                               responseMock,
                                                               environ,
@@ -908,8 +920,12 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         state        = "state"
         expected_redirect_location = "%s#error=%s" % (redirect_uri, error_code)
         
+        client_mock = Mock(Client)
+        client_mock.redirect_uris = [redirect_uri]
+        client_mock.has_redirect_uri.return_value = True
+        
         client_store_mock = Mock(spec=ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {"redirect_uris": [redirect_uri]}
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         error_mock = Mock(spec=OAuthUserError)
         error_mock.error = error_code
@@ -931,7 +947,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Location",
                                                     expected_redirect_location)
-        self.assertEqual(altered_response.status_code, "302 Moved Temporarily")
+        self.assertEqual(altered_response.status_code, 302)
         self.assertEqual(altered_response.body, "")
 
 class ResourceOwnerGrantTestCase(unittest.TestCase):
@@ -1003,15 +1019,14 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         handler.client_id = client_id
         result = handler.process(request_mock, response_mock, {})
         
-        site_adapter_mock.authenticate.assert_called_with(request_mock, {})
+        site_adapter_mock.authenticate.assert_called_with(request_mock, {},
+                                                          scopes)
         token_generator_mock.generate.assert_called_with()
-        access_token_store_mock.save_token.assert_called_with(client_id,
-                                                              access_token,
-                                                              scopes,
-                                                              user)
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
         response_mock.add_header.assert_called_with("Content-Type",
                                                     "application/json")
-        self.assertEqual(result.status_code, "200 OK")
+        self.assertEqual(result.status_code, 200)
         self.assertEqual(json.loads(result.body), expected_response_body)
         self.assertEqual(result, response_mock)
     
@@ -1043,7 +1058,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         token_generator_mock.generate.assert_called_with()
         response_mock.add_header.assert_called_with("Content-Type",
                                                     "application/json")
-        self.assertEqual(result.status_code, "200 OK")
+        self.assertEqual(result.status_code, 200)
         self.assertDictEqual(expected_response_body, json.loads(result.body))
         self.assertEqual(result, response_mock)
     
@@ -1053,8 +1068,11 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         password      = "johnpw"
         username      = "johndoe"
         
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret
+        
         client_store_mock = Mock(ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {"secret": client_secret}
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(Request)
         request_mock.post_param.side_effect = [client_id, client_secret,
@@ -1099,7 +1117,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         client_id = "abcd"
         
         client_store_mock = Mock(ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = None
+        client_store_mock.fetch_by_client_id.side_effect = ClientNotFoundError
         
         request_mock = Mock(Request)
         request_mock.post_param.return_value = client_id
@@ -1123,8 +1141,11 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         client_secret_actual = "foo"
         client_secret_expected = "xyz"
         
+        client_mock = Mock(Client)
+        client_mock.secret = client_secret_expected
+        
         client_store_mock = Mock(ClientStore)
-        client_store_mock.fetch_by_client_id.return_value = {"secret": client_secret_expected}
+        client_store_mock.fetch_by_client_id.return_value = client_mock
         
         request_mock = Mock(Request)
         request_mock.post_param.side_effect = [client_id, client_secret_actual]
