@@ -11,7 +11,7 @@ class AuthorizationControllerTestCase(unittest.TestCase):
         self.token_generator_mock = Mock()
         
         self.auth_server = AuthorizationController(access_token_store=Mock(),
-                                               auth_token_store=Mock(),
+                                               auth_code_store=Mock(),
                                                client_store=self.client_store_mock,
                                                site_adapter=Mock(),
                                                token_generator=self.token_generator_mock,
@@ -57,6 +57,6 @@ class AuthorizationControllerTestCase(unittest.TestCase):
         
         response_mock.add_header.assert_called_with("Content-type",
                                                     "application/json")
-        self.assertEqual(response_mock.status_code, "400 Bad Request")
+        self.assertEqual(response_mock.status_code, 400)
         self.assertEqual(response_mock.body, json.dumps(error_body))
         self.assertEqual(result, response_mock)
